local gr,pi,sin,cos = love.graphics,math.pi,math.sin,math.cos
local pi2,pi4 = pi/2,pi/4
local path = {}
-- path (a x,y table) defined by a string of characters
-- functions change each chars into an angles used to calculate a new locations
-- character directions:
	-- 90 degrees: u=up d=down l=left r=right
	-- 45 degrees: q=ul w=ur a=dl s=dr
-- characters inside brackets are repeated
-- e.g. a square is defined by '4[r]' or 4[l]		=>	'rrrr' and 'llll'
-- Note: only a single leading number can be used 1-9 but not 12 etc
-- the distance between any two points is fixed (sz=10) for a figure.
-- {might want to make diagonal distances = 1.414*sz???)

path.ang = { u = 0, r = pi2, d = pi, l = -pi2,	-- converts chars to angles
			    q =-pi4, w = pi4, a = 3*pi4, s = -3*pi4 }

path.calc = function( x, y, str, sz )
	local str = path.exp( str )					-- linearize loops
	local x2, y2 , a = x, y, 0
	local p = { tk = 0, st = 1 }					-- for animation (time, step in path)
	for i = 1, #str do
		x2, y2, a = path.newpoint( str:sub( i, i ), x2, y2, sz, a )
		p[#p+1] = { x2, y2, a }
		end
	return p
	end

path.newpoint = function( an, x, y, ln, dir )	-- motion relative current direction
	local a = path.ang[an] + dir						-- facing angle
	local lx = ln * cos( a-pi2 ) + x
	local ly = ln * sin( a-pi2 ) + y
	return lx, ly, a
	end

path.exp = function( s )								-- expand loops
	local inner = s:match( "%[(%a+)%]" )			-- content within brackets - 1st pair
	if inner == nil then return s						-- expanded pairs #[inner]
		else
		local where = s:find("%d%[(%a+)%]")			 -- location of leading number
		local num	= s:sub( where, where )			 -- get number of reps
		local mult	=	string.rep( inner, num )	 -- expanded content in [inner]
		s = string.format('%s%s',s:sub(1,where-1),s:sub(where+1,#s)) -- remove number
		s = s:gsub('%['..inner..'%]',mult,1)		 -- replace inner with mult
		s = path.exp( s )									 -- see if there are other []'s
		end
	return s
	end

path.update = function( p, t, delay )		-- movement - (path and frame time)
	local delay = delay or 0.2					-- default delay = 0.2 seconds
	p.tk = p.tk + t
	if p.tk > delay then
		p.tk = 0
		p.st = p.st < #p and p.st + 1 or 1	-- next step along path
		end
	end

path.animate = function( p, clr )
	if clr then			-- without clr no following path displayed. Only arrow.
		gr.setColor( clr )
		for i = 1, p.st-1 do									-- color change = path traversed
			gr.line( p[i][1], p[i][2], p[i+1][1], p[i+1][2] )
			end
		end
	gr.setColor(1,1,1)
	if p.st < #p then
		imageAngle = path.angle( p[p.st], p[p.st+1] )	-- angle to next point
		else
		imageAngle = path.angle( p[1], p[2] )
		end
		gr.draw( arrow, p[p.st][1], p[p.st][2], imageAngle, .08, .05, 0, arrow:getHeight()/2 )
		--end
	end

path.pointing = function( pt )	-- direction of movement (for arrow)
	local l = 8
	local x = l * cos( pt[3]-pi2 ) + pt[1]
	local y = l * sin( pt[3]-pi2 ) + pt[2]
	gr.line( pt[1], pt[2], x, y )
	end

path.angle = function( p1, p2 )
	local dx,dy = p2[1]-p1[1], p2[2]-p1[2]
	return math.atan2(dy,dx)
	end

path.draw = function( p, clr )
	gr.setColor( clr )
	for i = 1, #p do
		gr.line( p[i][1], p[i][2], p[i%(#p)+1][1], p[i%(#p)+1][2] )
		end
	end

path.rotatePoint = function( px, py, an, cx, cy )	-- p rotated around c
	local c, s, dx, dy = cos( an ), sin( an ), px-cx, py-cy
	return c * dx - s * dy + cx, s * dx + c * dy + cy
	end

-- ***** simply draw path at new location (rotated and scaled)
path.Draw = function( p, x, y, an, sx, sy, clr )	-- rotate around center
	gr.setColor( clr or {1,1,1} )
	local sx, sy,x1, y1, x2, y2  = sx or 1, sy or 1
	local cum = { 0, 0 }	-- find center
	for i = 1, #p do cum[1] = cum[1]+p[i][1] cum[2] = cum[2]+p[i][2] end
	local cm = { cum[1]/#p, cum[2]/#p }	-- center of mass
	for i = 1, #p-1 do
		x1,y1 = path.rotatePoint( p[i][1]-cm[1], p[i][2]-cm[2], an, 0, 0 )
		x2,y2 = path.rotatePoint( p[i+1][1]-cm[1], p[i+1][2]-cm[2], an, 0, 0 )
		gr.line( sx*x1 + x, sy*y1 + y, sx*x2 + x, sy*y2 + y )
		end
	end

-- ***** creates new path from: path, new location, rotation, scale
path.edit = function( p, x, y, a, sx, sy )
	local sx, sy, a, x1, y1  = sx or 1, sy or 1, a or 0
	local mp = { tk=p.tk, st=p.st }	-- for animation - lapsed time, index into table
	local cum = { 0, 0 }	-- find rotation center (of mass)
	for i = 1, #p do cum[1] = cum[1]+p[i][1] cum[2] = cum[2]+p[i][2] end
	local cm = { cum[1]/#p, cum[2]/#p }	-- center of mass
	for i = 1, #p do		-- zero 'a' just translates object to x,y
		x1,y1 = path.rotatePoint( p[i][1]-cm[1], p[i][2]-cm[2], a, 0, 0 )
		mp[#mp+1] = { sx*x1+x, sy*y1+y, p[i][3]+a }		-- scale and rotate around x,y
		end
	return mp 		-- return modified path
	end

path.drawtext = function(n)
	gr.print( n,		loc[n][1] + textof[n][1], loc[n][2] + textof[n][2] )
	gr.print( ln[n],	loc[n][1] + textof[n][3], loc[n][2] + textof[n][4] )
	end

path.linearFormat = function( self )
	-- convert {{x1,y1},{x2,y2}...} into {x1,y1,x2,y2, ...} (for use with polygon)
	local tab = {}
	for i = 1, #self do
		tab[#tab+1] = self[i][1]
		tab[#tab+1] = self[i][2]
		end
	return tab
	end

path.fillConcave = function( self, clr )
	gr.setColor(clr or {1,1,1})		-- <= not sure this should be here!
	local tri = love.math.triangulate( path.linearFormat( self ) )
	for i = 1, #tri do gr.polygon( 'fill', unpack( tri[i] ) ) end
	end

return path
